"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readLambdaConfig = exports.RouteType = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
var RouteType;
(function (RouteType) {
    RouteType["DIR"] = "DIR";
    RouteType["FUNCTION"] = "FUNCTION";
})(RouteType = exports.RouteType || (exports.RouteType = {}));
function readLambdaConfig(dir) {
    const result = readLambdaConfigImpl(dir, dir);
    if (!result.children) {
        throw new Error('Invalid directory ' + dir);
    }
    return flattenConfig(result.children).filter((e) => e.type === RouteType.FUNCTION);
}
exports.readLambdaConfig = readLambdaConfig;
function flattenConfig(config) {
    const arr = [];
    for (const child of Object.entries(config)) {
        const newVal = { ...child[1] };
        newVal.children = undefined;
        arr.push(newVal);
        if (child[1].children) {
            arr.push(...flattenConfig(child[1].children));
        }
    }
    return arr;
}
function posixPath(pathstring) {
    return pathstring.split(path_1.sep).join(path_1.posix.sep);
}
function removeExtension(path) {
    return path.replace(/\.[^/.]+$/, '');
}
function makePath(configRoot, dir) {
    const path = removeExtension(posixPath((0, path_1.relative)(configRoot, dir)));
    if (path === '$index') {
        return '/';
    }
    if (path.indexOf('$index') !== -1) {
        let newPath = path.replace('$index', '');
        // API Gateway does not accept routes like /myroute/
        newPath = newPath.slice(0, newPath.length - 1);
        return `/${newPath}`;
    }
    if (path === '$default') {
        return '$default';
    }
    return `/${path}`;
}
function makeRoute(configRoot, dir) {
    const route = makePath(configRoot, dir);
    if (route === '$default') {
        return '$default';
    }
    return `ANY ${route}`;
}
function readLambdaConfigImpl(configRoot, dir) {
    const root = dir;
    const rootRoute = makeRoute(configRoot, dir);
    const configItem = {
        name: root,
        type: RouteType.DIR,
        absoluteFilePath: dir,
        path: makePath(configRoot, dir),
        relativeFilePath: (0, path_1.relative)(configRoot, dir),
        route: rootRoute,
    };
    const dirHandle = fs_1.default.opendirSync(dir);
    let dirEntry;
    while ((dirEntry = dirHandle.readSync()) !== null) {
        if (!configItem.children) {
            configItem.children = {};
        }
        const name = dirEntry.name;
        const fullPath = (0, path_1.resolve)(root, name);
        if (dirEntry.isDirectory()) {
            configItem.children[name] = readLambdaConfigImpl(configRoot, fullPath);
        }
        else if (dirEntry.isFile()) {
            const route = makeRoute(configRoot, fullPath);
            configItem.children[name] = {
                name: removeExtension(name),
                type: RouteType.FUNCTION,
                path: makePath(configRoot, fullPath),
                absoluteFilePath: fullPath,
                relativeFilePath: (0, path_1.relative)(configRoot, fullPath),
                route: route,
            };
        }
        else {
            continue;
        }
    }
    dirHandle.closeSync();
    return configItem;
}
//# sourceMappingURL=collectLambdasFromFiles.js.map